package client.ui;

/* This file is Copyright by Walter W. Schilling, Jr. in the year 2004.
 * All rights of usage are limited. */

/**
 * @author Walter Schilling
 * $Author: wws $
 * $Revision: 1.9 $
 * $Date: 2004/04/27 22:49:56 $
 * $Header: /cvs/java_lab6/ChatGUIPanel.java,v 1.9 2004/04/27 22:49:56 wws Exp $
 * $Name:  $
 * This class implements the main panel for the Simple text chat client.  
 * This includes the entire GUI, etc.
 * 
 * $Log: ChatGUIPanel.java,v $
 * Revision 1.9  2004/04/27 22:49:56  wws
 * Updated javadoc representation of routines / methods to be consistent and relativly correct.
 *
 * Revision 1.8  2004/04/27 22:20:49  wws
 * Added support to shutdown the remote server when the last user disconnects from the server.
 * Performed minor refactoring of code.
 *
 * Revision 1.7  2004/04/20 22:18:26  wws
 * Implemented RMI methodology for accessing chat server.
 *
 * Revision 1.6  2004/04/18 13:35:08  wws
 * Changed interface so it naturally deals with rmi exceptions.
 *
 * Revision 1.5  2004/04/17 11:07:21  wws
 * Added capability to save the textual log of the chat session to a file.
 *
 * Revision 1.4  2004/04/17 10:56:09  wws
 * Modified GUI so that it operated in a polled, multithreaded manner.
 *
 * Revision 1.3  2004/04/16 21:30:45  wws
 * Added simplistic transmission and reception, as well as adding new button to spawn a second gui.  Transmissions can now occur between GUIs using basic chat protocol.
 *
 * Revision 1.2  2004/04/13 21:34:10  wws
 * Updated GUI to include support for enabling and disabling of the menu and lower buttons when conected and disconnected.
 *
 * Revision 1.1  2004/04/13 21:05:49  wws
 * Initial Chat Client GUI development.
 *
 *
 */

import javax.swing.*;

import common.DebugLog;

import client.ClientUser;

import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

/**
 * @author schilling
 * 
 */
public class ChatGUIPanel extends JPanel implements ActionListener {
	private JFileChooser saveFileChooser = new JFileChooser();

	public void setMyTitle(String title) {
		this.frame.setTitle(title);
	}

	private class ChatGUIUserPanel extends JPanel {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private JTextArea l_users = new JTextArea(10, 10);
		private JScrollPane l_users_scroller = new JScrollPane(l_users);

		public ChatGUIUserPanel() {
			super();
			l_users_scroller
					.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
			l_users_scroller
					.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
			add(l_users_scroller);
		}

		public void setUsersText(ArrayList<ClientUser> users) {
			this.removeAll();

			this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS)); // Set the
			// layout to
			// Box
			// layout.

			// this.setLayout(new GridLayout(users.size()+2, 1, 5, 5));
			// this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
			this.add(new JLabel("Current Users: "));
			this.add(new JLabel("(Checked users ignored)"));

			for (ClientUser currentuser : users) {
				this.add(new ClientUserVisualRepresentation(currentuser));

				// text += currentuser.getUserScreenName() + "\n";
			}

			this.validate();
			this.repaint();
		}
	}

	private class ChatGUILowerPanel extends JPanel {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private JTextField text_block;
		private JLabel block_label;
		private JButton transmit_button;

		/**
		 * This method will create an instance of the ChatGUILowerPanel.
		 * 
		 * @param p_ActionListernerImplementation
		 *            This is the action lister which is to listen for button
		 *            and menu events.
		 */
		public ChatGUILowerPanel() {
			setLayout(new BorderLayout());

			/* Create the buttons */
			transmit_button = new JButton("Transmit");
			block_label = new JLabel("Text to send: ");
			text_block = new JTextField(50);

			/* Add the buttons to the panel in the correct order. */
			add(block_label, BorderLayout.WEST);
			add(text_block, BorderLayout.CENTER);
			add(transmit_button, BorderLayout.EAST);

			/* Add the event listeners to the buttons. */
			transmit_button.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					String textToTransmit = get_text();
					controllerInstance.transmitMessage(textToTransmit);
				}
			});

			text_block.addKeyListener(new KeyListener() {
				public void keyPressed(KeyEvent arg0) {
					if (arg0.getKeyCode() == KeyEvent.VK_ENTER) {
						String textToTransmit = get_text();
						controllerInstance.transmitMessage(textToTransmit);
					}
				}

				public void keyReleased(KeyEvent arg0) {
				}

				public void keyTyped(KeyEvent arg0) {
				}
			});
		}

		/**
		 * This method will return the text in the GUI lower panel, namely that
		 * which the user has typed.
		 * 
		 * @return A string will be returned representing the text that the user
		 *         has typed.
		 */
		public String get_text() {
			return new String(text_block.getText());
		}

		/**
		 * This method will clear the text in the lower textbox to an empty
		 * string.
		 * 
		 */
		public void clear_text() {
			text_block.setText(new String(""));
		}

		/**
		 * This method will disable the panel when the panel should no longer be
		 * responsive to user stimuli.
		 * 
		 */
		public void disable_panel() {
			text_block.setEnabled(false);
			transmit_button.setEnabled(false);
		}

		/**
		 * This method will reanable the panel when it should again respond to
		 * user stimuli.
		 * 
		 */
		public void enable_panel() {
			text_block.setEnabled(true);
			transmit_button.setEnabled(true);
		}
	}

	private class Menus extends JPanel {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private JMenuBar l_bar;
		private JMenu l_fileMenu;
		private JMenuItem l_new_menu;
		private JMenuItem l_save_menu;
		private JMenuItem l_connect_menu;
		private JMenuItem l_disconnect_menu;
		private JMenu l_helpMenu;
		private JCheckBoxMenuItem l_soundsEnabled;

		/**
		 * The default constructor just sets up the menus and places them in a
		 * panel. There are only two menus and they are both "tied" to the left
		 * hand side of the panel.
		 */
		public Menus(ActionListener p_listener) {
			l_bar = new JMenuBar();
			setLayout(new FlowLayout(FlowLayout.LEFT));
			l_fileMenu = new JMenu("File");
			l_new_menu = new JMenuItem("New");

			l_new_menu.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					controllerInstance.CreateNewProgramInstance();
				}
			});

			l_save_menu = new JMenuItem("Save");

			l_connect_menu = new JMenuItem("Connect");
			l_connect_menu.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					controllerInstance.connectToServer();
				}
			});

			l_disconnect_menu = new JMenuItem("Disconnect");
			l_disconnect_menu.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					controllerInstance.disconnectFromServer();
				}
			});

			l_fileMenu.add(l_new_menu).addActionListener(p_listener);

			l_save_menu.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					controllerInstance.saveMessageLog();
				}
			});

			l_fileMenu.add(l_save_menu);

			l_fileMenu.add(l_connect_menu).addActionListener(p_listener);

			l_fileMenu.add(l_disconnect_menu).addActionListener(p_listener);

			// a group of check box menu items
			l_fileMenu.addSeparator();
			l_soundsEnabled = new JCheckBoxMenuItem("Sounds Enabled");
			l_soundsEnabled.setSelected(true);
			l_fileMenu.add(l_soundsEnabled);
			l_soundsEnabled.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent arg0) {
					AbstractButton aButton = (AbstractButton) arg0.getSource();
					boolean selected = aButton.getModel().isSelected();
					controllerInstance.setPlaySound(selected);
				}
			});

			l_fileMenu.addSeparator();

			(l_fileMenu.add(new JMenuItem("Quit")))
					.addActionListener(p_listener);
			l_helpMenu = new JMenu("Help");

			JMenuItem about = new JMenuItem("About");
			about.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					JOptionPane
							.showMessageDialog(
									getRootPane(),
									"EECS 7980: Special Topics: Java\nJava Chat Client\nAuthor: Walter Schilling\n$Date: 2004/04/27 22:49:56 $",
									"About", JOptionPane.INFORMATION_MESSAGE,
									null);
				}
			});
			l_helpMenu.add(about);

			l_bar.add(l_fileMenu);
			l_bar.add(l_helpMenu);
			add(l_bar);
		}

		/**
		 * This method will be called when a conection should be allowed,
		 * setting the menu to the appropriate greyed out state.
		 */
		public void allow_connection() {
			l_connect_menu.setEnabled(true);
			l_disconnect_menu.setEnabled(false);
		}

		/**
		 * This method will be called when a conection should be dis-allowed,
		 * setting the menu to the appropriate greyed out state.
		 */
		public void allow_disconnection() {
			l_connect_menu.setEnabled(false);
			l_disconnect_menu.setEnabled(true);
		}
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private JFrame frame = new JFrame();
	private Menus menubar = new Menus(this);
	private JTextArea l_description_representation = new JTextArea(10, 50);
	private JScrollPane l_scroller = new JScrollPane(
			l_description_representation);
	private ChatGUILowerPanel l_lower_panel = new ChatGUILowerPanel();
	private ChatGUIUserPanel l_user_panel = new ChatGUIUserPanel();
	private JScrollPane userListingScrollPane = new JScrollPane(l_user_panel);

	private ChatGUIController controllerInstance = null;

	/**
	 * This constructor will construct a ChatGUIPanel.
	 * 
	 * @param t_interface
	 *            This is the interface for performing chat communications that
	 *            is to be used.
	 */
	public ChatGUIPanel(ChatGUIController controllerInstance) {
		this.controllerInstance = controllerInstance;
		setLayout(new BorderLayout());
		frame.setContentPane(this);
		frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.BuildPanel();
		l_lower_panel.disable_panel();
		menubar.allow_connection();
		frame.pack();
		frame.setVisible(true);
		frame.addWindowListener(new WindowListener() {

			public void windowActivated(WindowEvent arg0) {
				// Not interested.
			}

			@Override
			public void windowClosed(WindowEvent arg0) {
				// Not interested.
			}

			@Override
			public void windowClosing(WindowEvent arg0) {
				System.exit(0);
			}

			@Override
			public void windowDeactivated(WindowEvent arg0) {
				// Not interested
			}

			@Override
			public void windowDeiconified(WindowEvent arg0) {
				// Not interested

			}

			@Override
			public void windowIconified(WindowEvent arg0) {
				// Not interested

			}

			@Override
			public void windowOpened(WindowEvent arg0) {
				// not interested

			}
		});
	}

	/**
	 * This method will build the panel, placing all GUI objects on the panel.
	 * 
	 */
	public void BuildPanel() {
		add(menubar, BorderLayout.NORTH);
		l_scroller
				.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		l_scroller
				.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		add(l_scroller, BorderLayout.CENTER);
		add(l_lower_panel, BorderLayout.SOUTH);
		add(userListingScrollPane, BorderLayout.EAST);
	}

	/**
	 * This method is responsible for updating the text panel with new textual
	 * strings. Due to the multithreaded nature, this must be a thread safe
	 * routine, and thus, the synchronized definition.
	 */
	public void update_textbox_with_new_text(String newText) {
		this.l_description_representation.setText(newText);
	}

	/**
	 * This routine will be invoked to update the connection status on the
	 * display.
	 * 
	 * @param connectionDisplay
	 *            This parameter will be set to true if a connection has been
	 *            established and the connected display should be shown. If it
	 *            is false it indicates that a connection has not been
	 *            established.
	 */
	public void updateConnectionState(boolean connectionDisplay) {
		if (connectionDisplay == true) {
			l_lower_panel.enable_panel();
			menubar.allow_disconnection();
		} else {
			l_lower_panel.disable_panel();
			menubar.allow_connection();
		}
	}

	/**
	 * This methjod shall be invoked whenever an action occurs on the gui
	 * requiring handling.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("Quit")) {
			handle_quit_menu();
		}
		DebugLog.println("EVent: " + e.getActionCommand());
	}

	public void showErrorMessage(String text, String type) {
		JOptionPane.showMessageDialog(null, text, type,
				JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * This method will be invoked when the user selects the quit option from
	 * the menu.
	 */
	private void handle_quit_menu() {
		int result = JOptionPane.showConfirmDialog(this.getRootPane(),
				"Are you sure you want to quit?", "Quit",
				JOptionPane.YES_NO_OPTION);
		if (result == JOptionPane.YES_OPTION) {
			this.controllerInstance.quit();
		}
	}

	/**
	 * This method will be invoked when the user selects the disconnect menu
	 * option.
	 */
	public boolean showServerShutdownDisplay() {
		int result = JOptionPane
				.showConfirmDialog(
						this.getRootPane(),
						"You are the last user of the server. WOuld you like to shut it down?",
						"Shutdown Server", JOptionPane.YES_NO_OPTION);
		if (result == JOptionPane.YES_OPTION) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * This menu will be invoked when the user wishes to save a log of the
	 * textual transmissions.
	 * 
	 */
	public String getMessageLogFilename() {
		String filename = "";

		// Note: source for ExampleFileFilter can be found in FileChooserDemo,
		// under the demo/jfc directory in the Java 2 SDK, Standard Edition.
		int returnVal = saveFileChooser.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			filename = saveFileChooser.getSelectedFile().getAbsolutePath();
			DebugLog.println("You chose to save this file: " + filename);
		}

		return filename;
	}

	public String getServerName(String previousServer) {
		return JOptionPane.showInputDialog(this.getRootPane(),
				"Enter server name for chat session.", previousServer);
	}

	public String getUserName(String previousUserName) {
		return JOptionPane.showInputDialog(this.getRootPane(),
				"Enter username for chat session.", previousUserName);
	}

	public String getScreenName(String previousScreenName) {
		return JOptionPane.showInputDialog(this.getRootPane(),
				"Enter screenname for chat session.", previousScreenName);
	}

	public void updateUsersListing(ArrayList<ClientUser> users) {
		this.l_user_panel.setUsersText(users);
	}

	public void clearTextToTransmitEntry() {
		this.l_lower_panel.text_block.setText("");
	}
}