package common;
/* This file is Copyright by Walter W. Schilling, Jr. in the year 2004.
 * All rights of usage are limited. */

/**
 * @author Walter Schilling
 * $Author: wws $
 * $Revision: 1.3 $
 * $Date: 2004/04/27 22:49:56 $
 * $Header: /cvs/java_lab6/ChatTextualEntryQueryReturnClass.java,v 1.3 2004/04/27 22:49:56 wws Exp $
 * $Name:  $
 * This class represents the return structure used when the Java Chat Client 
 * gets information from the Java Chat server about newer text strings.  This 
 * is the return when the client polls the server for new chat text.
 * 
 * $Log: ChatTextualEntryQueryReturnClass.java,v $
 * Revision 1.3  2004/04/27 22:49:56  wws
 * Updated javadoc representation of routines / methods to be consistent and relativly correct.
 *
 * Revision 1.2  2004/04/27 22:20:49  wws
 * Added support to shutdown the remote server when the last user disconnects from the server.
 * Performed minor refactoring of code.
 *
 * Revision 1.1  2004/04/16 21:30:04  wws
 * Initial implementation of server and server interface.  Server currently runs locally as of right now.
 *
 *
 */


import java.io.*;
import java.util.*;



public class ChatMessageList extends ArrayList<ChatTextualEntry> implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Date l_chat_timestamp;
	private int l_user_count;

	/**
	 * This method will create a ChatTextualEntryQueryReturnClass.
	 */
	public ChatMessageList() {
		super();
		l_chat_timestamp = new Date();
		l_user_count = 0;	
	}

	/**
	 * This method will create a ChatTextualEntryQueryReturnClass. 
	 * @param p_date This is the timestamp of the last entry in this class.
	 * @param p_transmissions These are the transmissions that have occurred.
	 */
	public ChatMessageList(Date p_date, ArrayList<ChatTextualEntry> p_transmissions) {
		l_chat_timestamp = p_date;
		this.clear();
		this.addAll(p_transmissions);
		l_user_count = 0;	
	}


	/**
	 * This method will create a ChatTextualEntryQueryReturnClass. 
	 * @param p_date This is the timestamp of the last entry in this class.
	 * @param p_transmissions These are the transmissions that have occurred.
	 * @param p_user_count This is a count of the number of users connected to the server.
	 */
	public ChatMessageList(Date p_date, ArrayList<ChatTextualEntry> p_transmissions, int p_user_count) {
		l_chat_timestamp = p_date;
		this.clear();
		this.addAll(p_transmissions);
		l_user_count = p_user_count;	
	}

    /**
     * This method will get the timestamp for the transmission.
     * @return The timestamp is returned as a Date.
     */
    public Date get_timestamp()
    {
    	return l_chat_timestamp;
    }
    
    /**
     * This method will get the transmissions that are present in this class. 
     * @return An array list of textual transmissions will occur.
     */
    public ArrayList<ChatTextualEntry> get_chat_transmissions()
    {
    	return this;
    }
    
    /**
     * This method will return the count of users on the system.
     * @return The usercount will be returned as an int.
     */
    public int get_usercount()
    {
    	return l_user_count;
    }
	
	/**
	 * This method will set the timestamp for when the last return occurred.
	 * @param p_date The timestamp is set as a Date class.
	 */
	public void set_timestamp(Date p_date)
	{
		l_chat_timestamp = p_date;
	}
    
	/**
	 * This method will set the chat transmissions within this class.
	 * @param p_chat_transmissions These are the chat transmissions which have occurred.
	 */
	public void set_chat_transmissions(ArrayList<ChatTextualEntry> p_chat_transmissions)
	{
		this.clear();
		this.addAll(p_chat_transmissions);
	}
	
	/**
	 * This method will set the user count.
	 * @param p_user_count This is the user count as an int.
	 */
	public void set_usercount(int p_user_count)
	{
		l_user_count = p_user_count;
	}
	
	/**
	 * This method will convert this class to a String.
	 */
	public String toString()
	{
		String t_return_string = new String("");
		t_return_string += l_chat_timestamp+"\n";
		int t_index;
		t_index = 0;
		while (t_index < this.size())
		{
			t_return_string += ((ChatTextualEntry)(this.get(t_index))).toString();
			t_index++;
		}
		return t_return_string;
	}
}
