package common;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;

/* This file is Copyright by Walter W. Schilling, Jr. in the year 2004.
 * All rights of usage are limited. */

/**
 * @author Walter Schilling $Author: wws $ $Revision: 1.2 $ $Date: 2004/04/27
 *         22:49:56 $ $Header: /cvs/java_lab6/ChatTextualEntry.java,v 1.2
 *         2004/04/27 22:49:56 wws Exp $ $Name: $ This class implements a
 *         textual entry between a client and the server. It will be used to
 *         transmit a transaction to the server, and will also be used as a
 *         respoinse vehicle when a client queries for new textual entries.
 * 
 *         $Log: ChatTextualEntry.java,v $ Revision 1.2 2004/04/27 22:49:56 wws
 *         Updated javadoc representation of routines / methods to be consistent
 *         and relativly correct.
 * 
 *         Revision 1.1.1.1 2004/04/13 20:19:27 wws This is the initial
 *         importation of the Java Project 6, related to a simple Java chat
 *         client.
 * 
 * 
 */

public class ChatTextualEntry implements Serializable, ChatUserInterface,
		Cloneable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Date l_entry_timestamp;
	private ChatUserInterface l_username;
	private String l_text;

	/**
	 * This is the default, empty constructor.
	 */
	public ChatTextualEntry() {
		l_entry_timestamp = new Date();
		l_username = null;
		l_text = new String("");
	}

	/**
	 * This constructor will be used when a new ChatTextualEntry class is to be
	 * created.
	 * 
	 * @param p_timestamp
	 *            This is the timestamp that is to be applied to the textual
	 *            entry.
	 * @param p_user
	 *            This is the user name who has entered the text.
	 * @param p_text
	 *            This is the text that is to be transmitted.
	 */
	public ChatTextualEntry(Date p_timestamp, ChatUserInterface p_user,
			String p_text) {
		l_entry_timestamp = (Date) p_timestamp.clone();
		l_username = p_user;
		l_text = new String(p_text);
	}

	/**
	 * This constructor will be used when a new ChatTextualEntry class is to be
	 * created.
	 * 
	 * @param p_user
	 *            This is the user name who has entered the text.
	 * @param p_text
	 *            This is the text that is to be transmitted.
	 */
	public ChatTextualEntry(ChatUserInterface p_user, String p_text) {
		l_entry_timestamp = new Date();
		l_username = p_user;
		l_text = new String(p_text);
	}

	/**
	 * This method will return the timestamp for this textual entry.
	 * 
	 * @return The timestamp will be returned as a Date class.
	 */
	public Date get_entry_timestamp() {
		return ((Date) l_entry_timestamp.clone());
	}

	/**
	 * This method will return the username given in the class.
	 * 
	 * @return The username will be returned as a string.
	 */
	public String get_username() {
		return l_username.getUserScreenName();
	}

	/**
	 * This method will get the text from the class.
	 * 
	 * @return The text entered is returned as a String.
	 */
	public String get_text() {
		return new String(l_text);
	}

	/**
	 * This method will set the timestamp.
	 * 
	 * @param p_entry_timestamp
	 *            The timestamp is passed in as a Date class.
	 */
	public void set_entry_timestamp(Date p_entry_timestamp) {
		l_entry_timestamp = (Date) p_entry_timestamp.clone();
	}

	/**
	 * This method will set the text.
	 * 
	 * @param p_text
	 *            This is the text that is to be transmitted.
	 */
	public void set_text(String p_text) {
		if (p_text == "") { // DO nothing

		} else {
			l_text = p_text;
		}
	}

	/**
	 * This method will convert this class into a textual string.
	 */
	public String toString() {
		String t_return_string = new String("");
		t_return_string += "[" + l_username.getUserScreenName() + " ";

		SimpleDateFormat t_date_format = new SimpleDateFormat(
				"M/d/yyyy H:mm:ss a");
		t_return_string += t_date_format.format(this.l_entry_timestamp);
		t_return_string += "]: " + this.l_text + "\n";
		return t_return_string;
	}

	@Override
	public String getUserName() {
		return this.l_username.getUserName();
	}

	@Override
	public String getUserScreenName() {
		return this.l_username.getUserScreenName();
	}

}
