package server;

/* This file is Copyright by Walter W. Schilling, Jr. in the year 2004.
 * All rights of usage are limited. */

/**
 * @author Walter Schilling
 * $Author: wws $
 * $Revision: 1.3 $
 * $Date: 2004/04/27 22:49:56 $
 * $Header: /cvs/java_lab6/NetworkJavaServer.java,v 1.3 2004/04/27 22:49:56 wws Exp $
 * $Name:  $
 * This class implements a network Java server.  It is accessed through an RMI.
 * 
 * $Log: NetworkJavaServer.java,v $
 * Revision 1.3  2004/04/27 22:49:56  wws
 * Updated javadoc representation of routines / methods to be consistent and relativly correct.
 *
 * Revision 1.2  2004/04/27 22:20:49  wws
 * Added support to shutdown the remote server when the last user disconnects from the server.
 * Performed minor refactoring of code.
 *
 * Revision 1.1  2004/04/20 22:18:26  wws
 * Implemented RMI methodology for accessing chat server.
 *
 *
 */

import java.util.*;
import java.rmi.*;
import java.rmi.server.*;

import common.ChatTextualEntry;
import common.ChatMessageList;
import common.ChatUserInterface;
import common.DebugLog;
import common.DuplicateUserException;

/**
 * This class simply implements relay functions into an instance of a
 * LocalJavaServer, the class instance l_local_server.
 */
public class NetworkJavaServer extends java.rmi.server.UnicastRemoteObject
		implements ChatServerClientInterface {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private LocalJavaServer l_local_server;

	public NetworkJavaServer() throws java.rmi.RemoteException {
		l_local_server = new LocalJavaServer();
	}

	public ChatMessageList login_user(ChatUserInterface p_username)
			throws java.rmi.RemoteException, DuplicateUserException {
		return l_local_server.login_user(p_username);
	}

	public void shutdown_server() {
		l_local_server.shutdown_server();
	}

	public ChatMessageList logoff_user(ChatUserInterface p_username)
			throws java.rmi.RemoteException {
		return l_local_server.logoff_user(p_username);
	}

	public ChatMessageList obtain_new_text_strings(Date p_previous_timestamp)
			throws java.rmi.RemoteException {
		return l_local_server.obtain_new_text_strings(p_previous_timestamp);
	}

	public void send_chat_text(ChatTextualEntry p_entry)
			throws java.rmi.RemoteException {
		l_local_server.send_chat_text(p_entry);
	}

	/**
	 * This routine will set up the RMI services on the java server.
	 * 
	 * @param args
	 *            These are the command line arguments. Currently, none are
	 *            used.
	 */
	public static void main(String[] args) {

		DebugLog.println("Attempting to start the Java Chat server services.");

		try {
			NetworkJavaServer t_chat_server = new NetworkJavaServer();

			RemoteRef location = t_chat_server.getRef();
			DebugLog.println(location.remoteToString());

			String registration = "RMIChat1";

			Naming.rebind(registration, t_chat_server);
			DebugLog.println("Service started as: " + registration);
		} catch (RemoteException rex) {
			rex.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public int getUserCount() {
		return this.l_local_server.getUserCount();
	}

	@Override
	public Collection<ChatUserInterface> getUserList() {
		return this.l_local_server.getUserList();
	}

	@Override
	public void checkinWithServer(ChatUserInterface user) {
		this.l_local_server.checkinWithServer(user);

	}
}
